/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.monitoring.core;


import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.hibernate.Session;
import org.spagic.metadb.base.ProcessInstanceStateConstants;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.dbutils.HibernateUtil;
import org.spagic.metadb.model.Attribute;
import org.spagic.metadb.model.LogJournal;
import org.spagic.metadb.model.PoolInstance;
import org.spagic.metadb.model.PoolType;
import org.spagic.metadb.model.Process;
import org.spagic.metadb.model.ProcessInstance;
import org.spagic.metadb.model.RelevantData;
import org.spagic.metadb.model.RelevantDataProcessInstance;
import org.spagic.metadb.model.RelevantDataProcessInstanceId;
import org.spagic.metadb.model.Rule;
import org.spagic.metadb.model.Step;
import org.spagic.metadb.model.TransitionInstance;
import org.spagic.metadb.model.TransitionState;
import org.spagic.metadb.model.XmlNamespace;
import org.spagic.monitoring.core.MonitorIService.TransitionManagementState;
import org.spagic.monitoring.message.MessageIFace;

public abstract class MonitorServiceBase implements MonitorIService {

	public static final String EXISTS_VARIABLE_REG_EX = ".*\\$<{1}.*>{1}.*";
	public static final String VARIABLE_REG_EX = "\\$<{1}.*>{1}";
	
	private static Map<String, String> namespaceURI = null;
	protected Log log = null;

	@SuppressWarnings("unchecked")
	public synchronized static void initNameSpaceURI(){
		Session aSession = null;
		try {
			namespaceURI = new HashMap<String, String>();
			aSession = HibernateUtil.getSessionFactory().openSession();
			List<XmlNamespace> lNs = aSession.createQuery(" from org.spagic.metadb.model.XmlNamespace").list();
			for (XmlNamespace ns : lNs) {
				namespaceURI.put(ns.getPrefix(), ns.getValue());
			}
		}catch(Throwable t){
			t.printStackTrace();
		}finally{
			aSession.close();
		}
	}

	public static Map<String, String> getNamespaceURI(){
		if (namespaceURI == null)
			initNameSpaceURI();
		return namespaceURI;
	}

	/**
	 * Manage the case when a new state for an existing transition has to be determined.
	 * @param message Current message
	 * @param transitionInstance Existing transition instance
	 * @param communicationPattern
	 * @param aSession Hibernate session
	 * @return New state for transition
	 */
	protected TransitionManagementState manageExistingTransition(MessageIFace message, TransitionInstance transitionInstance) {
		// There is already a Transition with this id logged: we
		// should update the existing row with the state of exchange
		TransitionManagementState newTransitionState = message.nextState(transitionInstance);
		return newTransitionState;
	}
	
	/**
	 * Trace relevant data for current message
	 * @param messageInfo Current message
	 * @param aSession Hibernate session
	 * @return List of RelevantData
	 * @throws Exception
	 */
	protected List<RelevantData> traceRelevantData(LogJournal messageInfo, Session aSession) 
		throws Exception {
		// 1. Retrieve all rules configured for current endpoint
		// The rules should be ordered by order_id
		// 2. Filter valid rules by date
		// 3. Apply each rule to produce relevant data
		
		List<RelevantData> result = new ArrayList<RelevantData>();
		
		// Retrieve current target endpoint
		TransitionInstance transitionInstance = messageInfo.getTransitionInstance();
		Step targetStep = transitionInstance.getStepByIdStepTarget();
		// Retrieve rules for target endpoint: the rules are
		// already filtered by date
		debug("traceRelevantData -> Get Rules For Enpoint ID ["+ targetStep.getIdStep()+"] name ["+targetStep.getName()+"]" );
		Collection<Rule> rules = AuditDBUtils.getRulesByStep(aSession, targetStep, messageInfo.getTransitionState());
		
		debug("traceRelevantData -> Found ["+rules.size()+"] rules for Step with id ["+ targetStep.getIdStep()+"] and name ["+targetStep.getName()+"]" );
		// 3. Apply each rule to produce relevant data
		for (Rule currentRule : rules) {			
			// Create a DOM node from XML text
			Document document = DocumentHelper.parseText(messageInfo.getLog());
			//Node node = document.selectSingleNode(currentRule.getExpr());
			
			String rdRegExpr = currentRule.getExpr();
			if (rdRegExpr.matches(EXISTS_VARIABLE_REG_EX)) {
				int beginVarName = rdRegExpr.indexOf("$<") + 2;
				int endVarName = rdRegExpr.indexOf(">", beginVarName);
				String varName = rdRegExpr.substring(beginVarName, endVarName);
				debug("Changing rule '" + rdRegExpr + "'");
				rdRegExpr = rdRegExpr.replaceFirst(VARIABLE_REG_EX, "/VARIABLES/VARIABLE[@name='" + varName + "']/VALUE");
				debug("into rule '" + rdRegExpr + "'");
			}
			XPath xPathExpression = new DefaultXPath(rdRegExpr);
			xPathExpression.setNamespaceURIs(getNamespaceURI());
			String extractedData = xPathExpression.valueOf(document);
			debug("-> Applying Rule id["+currentRule.getIdRule()+"] Xpath -> "+ currentRule.getExpr());
			debug("-> To message " + messageInfo.getLog() + "");
			
			

			if ((extractedData == null) || (extractedData.trim().length() == 0)){
				//
				// NON GENERO UN'ECCEZZIONE 
				// Semplicemente non salvo i dati
				//
				//throw new IllegalStateException(" No data extracted.");
				log.warn("->Rule ["+ currentRule.getExpr() + "] Have Produced No Data");
			}
			debug("-> Rule ["+ currentRule.getExpr() + "] extract Extracted Data ["+extractedData+"]");
			RelevantData relevantData = new RelevantData();
			relevantData.setLogJournal(messageInfo);
			relevantData.setValue(extractedData);
			relevantData.setAttribute(currentRule.getAttribute());
			
			aSession.save(relevantData);
			aSession.flush();
			
			ProcessInstance processInstance = transitionInstance.getProcessInstance();
			RelevantDataProcessInstance relevantDataInstanceSa = 
				new RelevantDataProcessInstance(new RelevantDataProcessInstanceId(processInstance.getIdProcessInstance(), relevantData.getIdRelevantData()),
						relevantData, processInstance);
			aSession.save(relevantDataInstanceSa);
			
			result.add(relevantData);
		}
		return result;
	}
	
	protected LogJournal traceMessage(MessageIFace message,
			TransitionInstance transitionInstance, TransitionState exchangeState,
			Session aSession) throws Exception {
		String xmlMessage = message.getXMLMessage();

		LogJournal messageInfo = new LogJournal();
		messageInfo.setTransitionInstance(transitionInstance);
		messageInfo.setTransitionState(exchangeState);
		messageInfo.setDatelog(message.getTransitionCurrentStepDate());
		messageInfo.setLog(xmlMessage);

		aSession.save(messageInfo);

		return messageInfo;
	}
	
	protected ProcessInstance createProcessInstance(Process process, String correlationID, Session aSession) {
		// This is a new process instance: create a new row
		// Before creating a new service assembly instance, I need
		// to retrieve
		// the service assembly, and to retrieve it I can use the
		// endpoint

		ProcessInstance processInstance = new ProcessInstance();
		processInstance.setProcess(process);
		Date now = new Date();
		processInstance.setStartdate(now);
		processInstance.setEnddate(now);
		processInstance.setIdCorrelation(correlationID);
		processInstance.setState(ProcessInstanceStateConstants.PROCESS_ACTIVE);

		Long idProcessSaved = (Long) aSession.save(processInstance);
		processInstance = (ProcessInstance) aSession.get(ProcessInstance.class, idProcessSaved);
		return processInstance;
	}

	protected void checkPoolCorrelation(LogJournal messageInfo, List<RelevantData> savedRelevantData, Session aSession) throws Exception {
		
		TransitionInstance transitionInstance = messageInfo.getTransitionInstance();
		ProcessInstance processInstance = transitionInstance.getProcessInstance();
		Process process = processInstance.getProcess();
		
		// Get all the iter type that have in the definition the Service Assembly of this instance
		List<PoolType> candidateIterTypes = AuditDBUtils.getCandidateIterTypesForProcess(process, aSession);
		
		String saName = processInstance.getProcess().getName();
		debug("checkIterTypeCorrelation -> Get the list of the iter types that the ["+saName+"] belongs to");
		
		PoolType theIterType = null;
		if (candidateIterTypes.size() > 0){
			if (candidateIterTypes.size() == 1){
				debug("checkIterTypeCorrelation -> Process ["+ saName +"] have only one iter type as candidate iter");
				theIterType = candidateIterTypes.get(0);
				debug("checkIterTypeCorrelation -> There's no need to apply rules the iter type is ["+theIterType.getName()+"]");
			}else{
				for (PoolType i : candidateIterTypes){
					debug("checkIterTypeCorrelation -> Process ["+ saName +"] is part of Iter Type ["+i.getName()+"]");
			
					if ( belongsToPoolType(messageInfo, i)){
						// The rule is verified 
						theIterType = i;
						break;
					}
				}
			}
		}else{
			debug("checkIterTypeCorrelation -> Process ["+ saName +"] have no candidate iter");
		}
		
		
		
		if (theIterType == null){
			debug("checkIterTypeCorrelation -> Iter Type not found");
			return;
		}else{
			debug("checkIterTypeCorrelation -> Found the Iter Type ["+theIterType.getName()+"]");
		}
		
		// We have the iterType
		
		if (AuditDBUtils.isFirstProcessInIterType(process, theIterType, aSession)){
			// New iter instance
			debug("checkIterTypeCorrelation -> Process["+process.getName()+"] is the first in Iter Type ["+theIterType.getName()+"] Defnition");
			debug("checkIterTypeCorrelation -> Try to create a new Iter Instance...");
			newPoolInstance(processInstance, theIterType,savedRelevantData,aSession);
			debug("checkIterTypeCorrelation -> Iter type created...");
		} else {
			// Existing iter instance
			debug("checkIterTypeCorrelation -> Process["+process.getName()+"] is the first in Iter Type ["+theIterType.getName()+"] Defnition");
			debug("checkIterTypeCorrelation -> Try to Relate this instance to another iter instance...");
			existingPoolInstance(processInstance, theIterType, savedRelevantData, aSession);
			debug("checkIterTypeCorrelation -> Iter type correlation finished");
		}

	}
	
	/**
	 * Checks if a message belongs to an existing iter type
	 * @param messageInfo Current message
	 * @param iterType Iter type to check
	 * @return True if the message belongs to the iter type
	 */
	private boolean belongsToPoolType(LogJournal messageInfo, PoolType iterType ){
		boolean result = false;
		try{
			String booleanXPathExpression = iterType.getRule().getExpr();
			debug("belongsToIterType -> Verify if the rule ["+booleanXPathExpression+"] applies to the message");
			debug(messageInfo.getLog());
			Document document = DocumentHelper.parseText(messageInfo.getLog());
			//DefaultXPath aDefaultXPath = new DefaultXPath(xPathExpression);
			//result =  aDefaultXPath.booleanValueOf(document);
			XPath xPathExpression = new DefaultXPath(booleanXPathExpression);
			xPathExpression.setNamespaceURIs(getNamespaceURI());
			result =  xPathExpression.booleanValueOf(document);
			debug("belongsToIterType -> Rule ["+booleanXPathExpression+"]" + ( result ? "Is Verified" : "Is Not Verified"));
		}catch (Exception ex) {
			log.error("belongsToIterType -> Exception occured ", ex );
			result = false;
		}
		
		return result;
	}
	
	
	/**
	 * Creates a new iter instance on database
	 * @param processInstance Service Assembly instance
	 * @param poolType Iter type of the iter instance to create
	 * @param savedRelevantDataList List of relevant data the compose the iter unique key
	 * @param aSession Hibernate session
	 */
	private void newPoolInstance(ProcessInstance processInstance, PoolType poolType, List<RelevantData> savedRelevantDataList, Session aSession) {
		// New iter instance
		// We have to create a new iter type instance and link it to this process instance
		PoolInstance poolInstance = new PoolInstance();
		
		Set<ProcessInstance> processInstances = new LinkedHashSet<ProcessInstance>(1);
		processInstances.add(processInstance);
		poolInstance.setProcessInstances(processInstances);
		poolInstance.setPoolType(poolType);
		
		String uniquePoolInstanceKey = computeUniquePoolInstanceKey(poolType, savedRelevantDataList, aSession);
		if (uniquePoolInstanceKey == null) {
			log.error("newIterInstance - > Cannot Create Pool Instance");
			return;
		}
		poolInstance.setUniquePoolInstanceKey(uniquePoolInstanceKey);
		aSession.save(poolInstance); 
		// Associate the Service Assembly instance to the iter instance
		processInstance.setPoolInstance(poolInstance);
		aSession.save(processInstance);
	
	}
	
	/**
	 * Computes a unique key for an iter, starting from a list of relevant data
	 * @param iterType Iter type of the iter instance
	 * @param savedRelevantDataList List of relevant data
	 * @param aSession Hibernate session
	 * @return The unique key for iter
	 */
	private String computeUniquePoolInstanceKey(PoolType iterType,  List<RelevantData> savedRelevantDataList, Session aSession){
		List<Attribute> attributesForIter = AuditDBUtils.getAttributeForIterType(iterType, aSession);
		StringBuffer sb = new StringBuffer();
		for (Attribute attribute : attributesForIter){
			boolean found = false;
			
			for (RelevantData rd : savedRelevantDataList){
				if (rd.getAttribute().getIdAttribute() == attribute.getIdAttribute()){
					sb.append("["+attribute.getIdAttribute()+":"+rd.getValue()+"]");
					found = true;
					break;
				}
			}
			if (!found)
				return null;
		}
		return ( sb.length() > 0 ? sb.toString() : null );
	}
	
	private void existingPoolInstance(ProcessInstance processInstance,
			PoolType iterType, List<RelevantData> savedRelevantDataList,
			Session aSession) throws Exception {

		String iterInstanceKey = computeUniquePoolInstanceKey(iterType,
				savedRelevantDataList, aSession);

		if (iterInstanceKey == null) {
			log.error("existingIterInstance -> This instance sa doe not belong to iter instance ");
			return;
		}

		PoolInstance poolInstance = AuditDBUtils.getPoolInstanceByUniqueKey(
				iterInstanceKey, iterType, aSession);
		if (poolInstance == null) {
			log.error("existingIterInstance -> This instance sa doe not belong to iter instance ");
			return;
		}

		// Now link the iter instance to the service assembly instance
		processInstance.setPoolInstance(poolInstance);
		aSession.save(processInstance);
	}

	public void debug(Object o){
		if (log.isDebugEnabled()){
			log.debug(o);
		}
	}

}
