/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.monitoring.message;


import java.util.Date;
import java.util.Set;

import javax.xml.namespace.QName;

import org.spagic.metadb.model.TransitionInstance;
import org.spagic.monitoring.core.MonitorIService.TransitionManagementState;

public interface MessageIFace {

	/**
	 * @return Unique identifier of the message
	 */
	public String getId();

	/**
	 * @return The native message, ie MessageExchange for JBI, SOAP message for BPEL
	 */
	public Object getRawMessage();

	/**
	 * @return The communication pattern: in-only, in-out, etc
	 */
	public String getCommunicationPattern();

	/**
	 * @return QName of source transition step
	 */
	public QName getSourceStepQName();
	
	/**
	 * @return QName of target transition step
	 */
	public QName getTargetStepQName();
	
	/**
	 * @return Id used for relating the messages to the process instance
	 */
	public String getCorrelationID();
	
	/**
	 * @return Sending date of the message
	 */
	public Date getTransitionStartDate();
	
	/**
	 * @return In case of multistep transitions, the date of current step
	 */
	public Date getTransitionCurrentStepDate();
	
	/**
	 * @return First transition state
	 */
	public TransitionManagementState getStartState();
	
	/**
	 * @return True if there is a message to trace.
	 */
	public boolean existsMessageToTrace();

	/**
	 * @return The byte array used for storing the message in the database.
	 */
	public byte[] getSerializedMessage();
	
	/**
	 * @return XML message
	 */
	public String getXMLMessage() throws Exception;
	
	/**
	 * Determine the next state for the current transition.
	 * @param transitionInstance Current transition instance
	 * @return The next state for the transition transitionInstance.
	 */
	public TransitionManagementState nextState(TransitionInstance transitionInstance);
	
	/**
	 * @return True if this message is the first one for the current transition
	 */
	public boolean isFirstMessageForTransition();
	
}
