/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.monitoring.smx.components.jms;


import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

import javax.jbi.messaging.MessageExchange;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.jms.JmsEndpoint;
import org.apache.servicemix.jms.standard.StandardConsumerProcessor;
import org.spagic.monitoring.core.jbi.MonitorServiceJBI;
import org.spagic.monitoring.smx.message.JBIMessage;

// Details on how to send ANY type of message on a JMS queue for ServiceMix, 
// please read the following link:
// http://www.nabble.com/MapMessage-and-or-ObjectMessage-tf2226788.html#a6170831

public class MonitorStandardConsumerProcessor extends StandardConsumerProcessor {
		
	private static final Log log = LogFactory.getLog(MonitorStandardConsumerProcessor.class);

	//	For a consumer endpoint, indicates if the processor
	//    should store the exchanges in the meta database, for
	//    using the restart feature.
	private boolean storeExchangesForRestart = true;
	
	//    For a consumer endpoint, indicates if the processor
	//    should keep, on the database, only the faulted exchanges
	//    (before the fault).
	//    If false, all ACTIVE exchanges are stored in the database,
	//    not only the faulted exchanges.
	private boolean keepOnlyFaultedExchanges = true;
	
	public MonitorStandardConsumerProcessor(JmsEndpoint ep) throws Exception {
		super(ep);
		loadRestartConfiguration();
	}

	@Override
	public void onMessage(Message message) {
		
		if (!(message instanceof ObjectMessage)) {
			log.warn("The message is not of type ObjectMessage");
			return;	
		}
		
		try {
			MessageExchange exchange = (MessageExchange)((ObjectMessage)message).getObject();
			JBIMessage jbiMessage = new JBIMessage(exchange, log);
			
	    	debug("traceExchange. Exchange = " + exchange);
	    	MonitorServiceJBI monitorService = new MonitorServiceJBI(storeExchangesForRestart, keepOnlyFaultedExchanges, log);
	    	monitorService.traceMessage(jbiMessage);
		}
		catch (JMSException ex) {
			log.error("Unable to extract Exchange from message", ex);
		}
	}
	
	public void debug(Object o){
		if (log.isDebugEnabled()){
			log.debug(o);
		}
	}
	
    /**
     * Loads the configuration for the management of exchanges for restart, from a properties
     * file stored inside the SU.
     * 
     */
    protected void loadRestartConfiguration() {
        String uri = "META-INF/monitorConfiguration";
        try {
            InputStream in = loadResource(uri);
            Properties props = new Properties();
            props.load(in);
            
        	storeExchangesForRestart = Boolean.parseBoolean(props.getProperty("storeExchangesForRestart"));
        	keepOnlyFaultedExchanges = Boolean.parseBoolean(props.getProperty("keepOnlyFaultedExchanges"));
        } catch (IOException e) {
            log.error("Could not find resource: " + uri);
        }
    }
    
    
    /**
     * Loads the resource from the given URI
     */
    protected InputStream loadResource(String uri) {
        // lets try the thread context class loader first
        InputStream in = Thread.currentThread().getContextClassLoader().getResourceAsStream(uri);
        if (in == null) {
            in = getClass().getClassLoader().getResourceAsStream(uri);
            if (in == null) {
                debug("Could not find resource: " + uri);
            }
        }
        return in;
    }
	
		
}
