/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-06-2008      Boccalon/Carbone         SPAGIC-131: Optimize the creation of the
* queue connection in the class AuditingExchangeASyncListener: it should performed 
* only once, since it supports concurrent use
* 										    
**/
package org.spagic.monitoring.smx.listeners;

import java.io.Serializable;
import java.util.Date;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.jms.ConnectionFactory;
import javax.jms.DeliveryMode;
import javax.jms.Destination;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.JbiConstants;
import org.apache.servicemix.jbi.event.ExchangeEvent;
import org.apache.servicemix.jbi.event.ExchangeListener;

public class AuditingExchangeASyncListener implements ExchangeListener {
	public static final String MEP_INONLY = "org.apache.servicemix.jbi.messaging.InOnlyImpl";
	public static final String MEP_INOUT = "org.apache.servicemix.jbi.messaging.InOutImpl";
	public static final String MEP_ROBUSTINONLY = "org.apache.servicemix.jbi.messaging.RobustInOnlyImpl";
	public static final String MEP_INOPTIONALOUT = "org.apache.servicemix.jbi.messaging.InOptionalOutImpl";

	private static final Log log = LogFactory.getLog(AuditingExchangeASyncListener.class);
	
	private ConnectionFactory connectionFactory = null;
	private Destination destinationQueue = null;
	private QueueConnection queueConnection = null;
	
	public AuditingExchangeASyncListener() {
		super();
	}
	
	public void debug(Object o){
		if (log.isDebugEnabled()){
			log.debug(o);
		}
	}
	
	private void createConnection() {
		try {
			// Close previous connection, if any
			if (queueConnection != null)
				queueConnection.close();
			
			// Create new connection
			debug("Creating Queue Connection.....");
        	queueConnection =
        		((QueueConnectionFactory)getConnectionFactory()).createQueueConnection();
        	queueConnection.start();
        	debug("Queue Connection is OK....");
		}
		catch (Exception ex) {
    		log.error(ex);
    	}
	}
	
	public ConnectionFactory getConnectionFactory() {
		return connectionFactory;
	}

	public void setConnectionFactory(ConnectionFactory connectionFactory) {
		this.connectionFactory = connectionFactory;
		// Create a connection with the ConnectionFactory
		createConnection();
	}

	public Destination getDestinationQueue() {
		return destinationQueue;
	}

	public void setDestinationQueue(Destination destinationQueue) {
		this.destinationQueue = destinationQueue;
	}

	
	public void exchangeSent(ExchangeEvent event) {
		 MessageExchange exchange = event.getExchange();
		 NormalizedMessage normalizedMessage = retrieveMessageToTrace(exchange);
		 if (normalizedMessage != null)
			 normalizedMessage.setProperty(JbiConstants.DATESTAMP_PROPERTY_NAME, new Date());
		 
		 QueueSession queueSession = null;
		 QueueSender queueSender = null;
    	
    	try {
        	debug("Creating Queue Session.......");
        	queueSession =
        		queueConnection.createQueueSession(false, javax.jms.Session.AUTO_ACKNOWLEDGE);
        	
        	debug("Creating Queue Sender....");
        	queueSender = queueSession.createSender((Queue)destinationQueue);
        	queueSender.setDeliveryMode(DeliveryMode.PERSISTENT);
        	
        	debug("Creating Object Message ");
        	ObjectMessage message = queueSession.createObjectMessage();
        	debug("Setting exchange in object message....");
        	message.setObject((Serializable)exchange);
        	
        	debug("Sending in queue............");
        	queueSender.send(message);
        	debug("Send OK ");
    	}
    	catch (Exception ex) {
    		log.error(ex);
    	}
    	finally {
    		try {
        		if (queueSender != null) {
                	queueSender.close();
        		}
        		if (queueSession != null) {
                	queueSession.close();
        		}
    		}
    		catch (Exception ex) {
    			log.error(ex);
    		}
    	}
    	
	}
	
	private NormalizedMessage retrieveMessageToTrace(MessageExchange exchange) {
		NormalizedMessage messageToTrace = null;
		String mep = exchange.getClass().getName();
		debug("Mep is:" + mep);
		if (mep.equals(MEP_INONLY)) {
			// The state can be only Active or Done
			debug(" MEP is IN_ONLY ");
			NormalizedMessage inMessage = ((InOnly) exchange).getInMessage();

			if (exchange.getStatus().equals(ExchangeStatus.ACTIVE) ||
				exchange.getStatus().equals(ExchangeStatus.DONE)) {
				messageToTrace = inMessage;
			} else {
				// Error
				Fault faultDetail = ((InOnly) exchange).getFault();
				Exception ex = ((InOnly) exchange).getError();
	        	debug("retrieveMessageToTrace -> Fault = " + faultDetail);
	        	log.debug("retrieveMessageToTrace ->", ex);
	        	
	        	if (faultDetail != null) {
					messageToTrace = faultDetail;
				} else if (ex != null) {
					messageToTrace = inMessage;
					log.error("Exchange with status ERROR: ", ex);
				} else {
					log.error("No error to trace: both getFault and getError return null");
				}
			}
		} else if (mep.equals(MEP_INOUT)) {
			debug(" MEP is IN_OUT ");
			if (exchange.getStatus().equals(ExchangeStatus.ACTIVE)) {
				
				// Check if there are errors: possible also if state is ACTIVE
				Fault faultDetail = ((InOut) exchange).getFault();
				if (faultDetail != null) {
					debug(" Message is FAULT");
					messageToTrace = faultDetail;
				}
				// I must check if this is the request or the response
				else if (exchange.getRole() == MessageExchange.Role.CONSUMER) {
					debug(" Role is CONSUMER ");
					NormalizedMessage outMessage = ((InOut) exchange).getInMessage();
					messageToTrace = outMessage;
				} else if (exchange.getRole() == MessageExchange.Role.PROVIDER) {
					debug(" Role is PROVIDER ");
					NormalizedMessage inMessage = ((InOut) exchange).getOutMessage();
					messageToTrace = inMessage;
				} else {
					// Error
		        	log.error("retrieveMessageToTrace. Mep type=MEP_INOUT, role unknown");
				}
			} else if (exchange.getStatus().equals(ExchangeStatus.DONE)) {
				NormalizedMessage inMessage = ((InOut) exchange).getInMessage();
				messageToTrace = inMessage;
			} else if (exchange.getStatus().equals(ExchangeStatus.ERROR)) {
				// Error
				Fault faultDetail = ((InOut) exchange).getFault();
				Exception ex = ((InOut) exchange).getError();
	        	debug("retrieveMessageToTrace -> Fault = " + faultDetail);
	        	log.debug("retrieveMessageToTrace ->", ex);
	        	
	        	if (faultDetail != null) {
					messageToTrace = faultDetail;
				} else if (ex != null) {
					log.error("Error creating error message", ex);
				} else {
					log.error("No error to trace: both getFault and getError return null");
				}
			} else {
				// Error
	        	log.error("retrieveMessageToTrace. Mep type=MEP_INOUT, status undefined ");
			}
		} else if (mep.equals(MEP_ROBUSTINONLY)) {
			debug(" MEP is ROBUST_IN_ONLY ");
		} else if (mep.equals(MEP_INOPTIONALOUT)) {
			debug(" MEP is ROBUST IN_OPTIONAL_OUT ");
		} else {
			// Errore
        	log.error("retrieveMessageToTrace. Mep type=UNDEFINED ");
		}
		return messageToTrace;
	}

	public void exchangeAccepted(ExchangeEvent event) {
		// Nothing to do
	}

}
