/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.monitoring.smx.listeners.debug;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;

import org.apache.servicemix.jbi.FaultException;
import org.apache.servicemix.jbi.event.ExchangeEvent;
import org.apache.servicemix.jbi.event.ExchangeListener;
import org.apache.servicemix.jbi.jaxp.SourceMarshaler;
import org.apache.servicemix.jbi.messaging.MessageExchangeImpl;

public class TraceFaultListener implements ExchangeListener {

	public static final Object lock = new Object();

	public static final String MEP_INONLY = "org.apache.servicemix.jbi.messaging.InOnlyImpl";

	public static final String MEP_INOUT = "org.apache.servicemix.jbi.messaging.InOutImpl";

	public static final String MEP_ROBUSTINONLY = "org.apache.servicemix.jbi.messaging.RobustInOnlyImpl";

	public static final String MEP_INOPTIONALOUT = "org.apache.servicemix.jbi.messaging.InOptionalOutImpl";

	public void exchangeSent(ExchangeEvent event) {
		// System.out.println(" ============================ Exchange accepted
		// ================================== ");
		MessageExchange exchange = event.getExchange();
		if (exchange instanceof MessageExchangeImpl == false) {
			throw new IllegalArgumentException(
					"exchange should be a MessageExchangeImpl");
		}
		try {
			synchronized (lock) {
				if (exchange.getStatus() == ExchangeStatus.ERROR) {
					Fault fault = null;
					Exception e = null;
					if (exchange instanceof InOnly) {
						e = exchange.getError();
						if (e instanceof FaultException) {
							fault = ((FaultException) e).getFault();
						}
					} else {
						fault = exchange.getFault();
						e = exchange.getError();
					}
					if (fault != null) {
						System.out
								.println("********** FAULT CONTENT **********");
						System.out.println(new SourceMarshaler().asString(fault
								.getContent()));
						System.out
								.println("***********************************");
					} else {
					    if ( e != null ){
					    	e.printStackTrace();
					    }
					}
				}

			}

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public TraceFaultListener() {
		super();
	}

	public String getMep(MessageExchange exchange) {
		String mepClassName = exchange.getClass().getName();
		return mepClassName;
	}

	public void traceMessage(NormalizedMessage message) {

	}

	public void exchangeAccepted(ExchangeEvent arg0) {
		// System.out.println(" ============================ Exchange sent
		// ================================== ");
	}

}
